"""Service Binding Port.

A service binding port is used to logically attach an edge port to a customer service using a :term:`VLAN`.
"""

from typing import Annotated

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle
from pydantic import Field

from gso.products.product_blocks.bgp_session import BGPSession, BGPSessionInactive, BGPSessionProvisioning
from gso.utils.shared_enums import SBPType
from gso.utils.types.ip_address import IPv4AddressType, IPv6AddressType

VLAN_ID = Annotated[int, Field(gt=0, lt=4096)]


class ServiceBindingPortInactive(
    ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="ServiceBindingPort"
):
    """A Service Binding Port that's currently inactive. See :class:`ServiceBindingPort`."""

    is_tagged: bool | None = None
    vlan_id: VLAN_ID | None = None
    sbp_type: SBPType | None = None
    ipv4_address: IPv4AddressType | None = None
    ipv6_address: IPv6AddressType | None = None
    custom_firewall_filters: bool | None = None
    geant_sid: str | None = None
    sbp_bgp_session_list: list[BGPSessionInactive] = Field(default_factory=list)


class ServiceBindingPortProvisioning(ServiceBindingPortInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A Service Binding Port that's currently being provisioned. See :class:`ServiceBindingPort`."""

    is_tagged: bool
    vlan_id: VLAN_ID | None = None
    sbp_type: SBPType
    ipv4_address: IPv4AddressType | None = None
    ipv6_address: IPv6AddressType | None = None
    custom_firewall_filters: bool
    geant_sid: str
    sbp_bgp_session_list: list[BGPSessionProvisioning]  # type: ignore[assignment]


class ServiceBindingPort(ServiceBindingPortProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """A service binding port that is actively used in the network."""

    #: Whether this :term:`VLAN` is tagged or not.
    is_tagged: bool
    #: The :term:`VLAN` ID.
    vlan_id: VLAN_ID | None = None
    #: Is this service binding port layer 2 or 3?
    sbp_type: SBPType
    #: If layer 3, IPv4 resources.
    ipv4_address: IPv4AddressType | None = None
    #: If layer 3, IPv6 resources.
    ipv6_address: IPv6AddressType | None = None
    #: Any custom firewall filters that the partner may require.
    custom_firewall_filters: bool
    #: The GÉANT service ID of this binding port.
    geant_sid: str
    #: The :term:`BGP` sessions associated with this service binding port.
    sbp_bgp_session_list: list[BGPSession]  # type: ignore[assignment]
